
library pub_bin.upgrade;

import 'dart:io';
import 'dart:async';

import 'package:path/path.dart';

import 'link.dart';

_upgradePackage(String base, String package) {
    return new Directory("$base/$package").exists().then((exists) {
        if (!exists)
            throw new Exception("Package $package is not installed");
        print("$package: Running pub get");
        return Process.start("pub", ["get"], workingDirectory: "$base/$package", environment: {'PUB_CACHE': "$base/.pub-cache"});
    })
    .then((process) {
        stdout.addStream(process.stdout);
        stderr.addStream(process.stderr);
        return process.exitCode;
    })
    .then((code) {
        if (code != 0)
            throw new Exception("Pub get ended with error $code");
        return link(base, package);
    });
}

upgrade(String base, List<String> packages) {
    if (packages.isEmpty) {
        Future previous = new Future.value();
        new Directory(base).list()
        .where((e) => e is Directory && !basename(e.path).startsWith('.'))
        .forEach((e) {
            previous = previous.then((_) {
                return _upgradePackage(base, basename(e.path)).catchError((e) {
                    print(e.toString());
                });
            });
        });
    }
    else {
        Future.forEach(packages, (package) {
            return _upgradePackage(base, package).catchError((e) {
                print(e.toString());
            });
        });
    }
}
